# flake8: NOQA
"""tmux session manager.

tmuxp
~~~~~

:copyright: Copyright 2013- Tony Narlock.
:license: MIT, see LICENSE for details

"""
from . import cli, config, util
from .__about__ import (
    __author__,
    __copyright__,
    __description__,
    __email__,
    __license__,
    __package_name__,
    __title__,
    __version__,
)
